from pyramid_who.whov2 import WhoV2AuthenticationPolicy
import gocept.directory2.admin.render
import gocept.directory2.application
import gocept.directory2.admin.auth
import pkg_resources


def get_groups(identity, request):
    """Compute the groups the currently logged-in user belongs to."""
    groups = []
    userid = identity['repoze.who.userid']
    user = gocept.directory2.user.HumanUser.query().filter_by(
        uid=userid).first()
    if user is not None and user.is_admin:
        groups.append('group:admin')
    return groups


class AdminApplication(gocept.directory2.application.ApplicationFactory):

    who_ini_filename = 'who.ini'

    def configure_auth(self):
        self.config.set_default_permission('admin')
        self.config.set_authentication_policy(
            WhoV2AuthenticationPolicy(
                pkg_resources.resource_filename(
                    __name__, self.who_ini_filename),
                'auth_tkt', get_groups))
        self.config.set_authorization_policy(
            gocept.directory2.admin.auth.GroupAdminAllowAllPolicy())

    def configure(self):
        super(AdminApplication, self).configure()
        self.configure_auth()
        config = self.config
        config.add_tween('gocept.directory2.admin.render.owrap_tween_factory',
                         under='pyramid_tm.tm_tween_factory')
        config.add_view_predicate(
            'owrap', gocept.directory2.admin.render.OwrapSelectionPredicate)

        config.include('pyramid_debugtoolbar')
        config.include('pyramid_deform')
        config.include('pyramid_fanstatic')
        config.include('deform_bootstrap')
        config.include('js.deform')
        # deform_bootstrap needs to be included after js.deform!
        config.include('js.deform_bootstrap')

        config.add_static_view(
            'resources', 'gocept.directory2.admin:resources',
            cache_max_age=3600)

        config.add_route('login', '/login')
        config.add_route('logout', '/logout')

        config.add_route('machine_list', '/machine/list')
        config.add_route('machine_add_physical', '/machine/add_physical')
        config.add_route('machine_add_virtual', '/machine/add_virtual')

        config.add_route('machine_view_physical', '/machine/physical/{id}',
                         factory='.machine.Node.context_factory')
        config.add_route('machine_view_virtual', '/machine/virtual/{id}',
                         factory='.machine.Node.context_factory')
        config.add_route('machine_edit_physical',
                         '/machine/physical/{id}/edit',
                         factory='.machine.PhysicalMachine.context_factory')
        config.add_route('machine_edit_virtual', '/machine/virtual/{id}/edit',
                         factory='.machine.VirtualMachine.context_factory')
        config.add_route('machine_confirm_delete',
                         '/machine/{id}/confirm-delete',
                         factory='.machine.Node.context_factory')
        config.add_route('machine_delete',
                         '/machine/{id}/delete',
                         factory='.machine.Node.context_factory')

        config.add_route('resourcegroup_list', '/resourcegroup/list')
        config.add_route('resourcegroup_add', '/resourcegroup/add')
        config.add_route(
            'resourcegroup_view', '/resourcegroup/{id}',
            factory='.resourcegroup.ResourceGroup.context_factory')
        config.add_route(
            'resourcegroup_edit', '/resourcegroup/{id}/edit',
            factory='.resourcegroup.ResourceGroup.context_factory')
        config.add_route(
            'resourcegroup_confirm_delete',
            '/resourcegroup/{id}/confirm-delete',
            factory='.resourcegroup.ResourceGroup.context_factory')
        config.add_route(
            'resourcegroup_delete',
            '/resourcegroup/{id}/delete',
            factory='.resourcegroup.ResourceGroup.context_factory')
        config.add_route(
            'resourcegroup_nodes', '/resourcegroup/{id}/nodes',
            factory='.resourcegroup.ResourceGroup.context_factory')

        config.add_route('permission_list', '/permission/list')
        config.add_route('permission_add', '/permission/add')
        config.add_route('permission_view', '/permission/{id}',
                         factory='.permission.Permission.context_factory')
        config.add_route('permission_edit', '/permission/{id}/edit',
                         factory='.permission.Permission.context_factory')
        config.add_route('permission_confirm_delete',
                         '/permission/{id}/confirm-delete',
                         factory='.permission.Permission.context_factory')
        config.add_route('permission_delete',
                         '/permission/{id}/delete',
                         factory='.permission.Permission.context_factory')

        config.add_route('location_list', '/location/list')
        config.add_route('location_add', '/location/add')
        config.add_route('location_view', '/location/{id}',
                         factory='.location.Location.context_factory')
        config.add_route('location_racks', '/location/{id}/racks',
                         factory='.location.Location.context_factory')
        # XXX Duplicate to location_view ???
        config.add_route('location', '/location/{id}',
                         factory='.location.Location.context_factory')
        config.add_route('location_edit', '/location/{id}/edit',
                         factory='.location.Location.context_factory')
        config.add_route('location_confirm_delete',
                         '/location/{id}/confirm-delete',
                         factory='.location.Location.context_factory')
        config.add_route('location_delete',
                         '/location/{id}/delete',
                         factory='.location.Location.context_factory')

        config.add_route('rack_view', '/racks/{id}',
                         factory='.rack.Rack.context_factory')

        config.add_route('user_list', '/user/list')
        config.add_route('user_view_human', '/user/human/{id}')
        config.add_route('user_add_human', '/user/add_human')

        config.add_route('user_view_service', '/user/service/{id}')
        config.add_route('user_add_service', '/user/add_service')

        config.add_route('report_list', '/report/list')
        config.add_route('traffic_by_customer', '/report/traffic_by_customer')
        config.add_route('traffic_by_customer_detail',
                         '/report/traffic_by_customer/{customer}')
        config.add_route('traffic_chart', '/report/traffic_chart')
        config.add_route('traffic_chart_data', '/report/traffic_chart/data')

        config.add_route('nic_view', '/nic/{id}',
                         factory='.nic.NIC.context_factory')

        config.add_route('domain_list', '/domain/list')
        config.add_route('domain_user_list', '/domain/user/list')
        config.add_route('domain_user_add', '/domain/user/add')
        config.add_route('domain_user_view', '/domain/user/{id}/',
                         factory='.domain.User.context_factory')
        config.add_route('domain_user_edit', '/domain/user/{id}/edit',
                         factory='.domain.User.context_factory')
        config.add_route('domain_list_for_user',
                         '/domain/user/{id}/list_domains',
                         factory='.domain.User.context_factory')
        config.add_route('domain_add_for_user',
                         '/domain/user/{id}/add_domain',
                         factory='.domain.User.context_factory')
        config.add_route('domain_add', '/domain/add')
        config.add_route('domain_view', '/domain/{id}',
                         factory='.domain.Domain.context_factory')
        config.add_route('domain_edit', '/domain/{id}/edit',
                         factory='.domain.Domain.context_factory')
        config.add_route('domain_confirm_delete',
                         '/domain/{id}/confirm-delete',
                         factory='.domain.Domain.context_factory')
        config.add_route('domain_delete',
                         '/domain/{id}/delete',
                         factory='.domain.Domain.context_factory')

        config.add_route('subdomain_list', '/domain/{id}/list',
                         factory='.domain.Domain.context_factory')
        config.add_route('subdomain_view', '/subdomain/{id}',
                         factory='.domain.Domain.context_factory')
        config.add_route('subdomain_edit', '/subdomain/{id}/edit',
                         factory='.domain.Domain.context_factory')
        config.add_route('subdomain_add', '/domain/{id}/add',
                         factory='.domain.Domain.context_factory')

        #The code does not need to know which route is used for the start page:
        config.add_route('home', '/')

        config.scan(package=__name__,
                    ignore=gocept.directory2.application.scan_ignore_tests)

factory = AdminApplication()
