from ..user import HumanUser
from pyramid.httpexceptions import HTTPForbidden, HTTPFound
from pyramid.security import NO_PERMISSION_REQUIRED
from pyramid.view import view_config, forbidden_view_config
import colander
import deform.widget
import gocept.directory2.admin.form
import pyramid.security


class LoginSchema(colander.Schema):
    username = colander.SchemaNode(colander.String())
    password = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.PasswordWidget())


@forbidden_view_config()
def forbidden_view(request):
    loc = request.route_url('login')
    return HTTPFound(location=loc)


@view_config(route_name='login',
             permission=NO_PERMISSION_REQUIRED)
class LoginView(gocept.directory2.admin.form.Form):
    buttons = ('login',)
    title = u'Login'
    schema = LoginSchema()

    def login_success(self, appstruct):
        login = appstruct.get('username')
        password = appstruct.get('password')
        user = HumanUser.query().filter_by(uid=login).first()
        if user is not None and user.check_password(password):
            headers = pyramid.security.remember(self.request, login)
            self.request.session.flash(u'Logged in.', 'success')
            return HTTPFound(location=self.request.route_url('home'),
                             headers=headers)
        self.request.session.flash(u'Login failed.', 'error')
        raise HTTPForbidden()


@view_config(route_name='logout')
def logout_view(request):
    headers = pyramid.security.forget(request)
    request.session.flash(u'Logged out.', 'success')
    return HTTPFound(location=request.route_url('home'), headers=headers)
