from deform.widget import TextInputWidget
import colander


class Relation(object):
    """Colander relation type."""

    def __init__(self, cls):
        self.cls = cls

    def serialize(self, node, appstruct):
        if appstruct is None or appstruct is colander.null:
            return colander.null
        # gocept.directory2.db.Object provides the primary key in ``id``.
        return str(appstruct.id)

    def deserialize(self, node, cstruct):
        if not cstruct:
            return colander.null
        return self.cls.find_by_id(cstruct)

    def cstruct_children(self):
        return []


class ReadOnlyNode(colander.SchemaNode):
    """Schema node for read only fields."""

    def __init__(self, *args, **kw):
        kw['missing'] = ''
        kw['widget'] = TextInputWidget(template='readonly/textinput')
        super(ReadOnlyNode, self).__init__(*args, **kw)


def is_ascii(node, value):
    """Validator which assures that value is ASCII."""
    try:
        value.encode('ascii')
    except UnicodeEncodeError:
        raise colander.Invalid(
            node, '"%s" is not ASCII, which is required here.' % value)
