from ..db import IDatabase
from ..resourcegroup import ResourceGroup
from ..traffic import Traffic
from .view import View, Action
from pyramid.response import Response
from pyramid.view import view_config
from sqlalchemy import func
from sqlalchemy.orm.exc import NoResultFound
import datetime
import json
import zope.component


@view_config(route_name='traffic_by_customer',
             renderer='templates/traffic_by_customer.pt')
class TrafficByCustomer(View):

    title = 'Traffic by customer'

    @property
    def customers(self):
        result = {}
        for resource_group in ResourceGroup.query():
            # XXX modelling customers would make this better
            result.setdefault(resource_group.customer_no, [])
            result[resource_group.customer_no].append(resource_group)
        return result


@view_config(route_name='traffic_by_customer_detail',
             renderer='templates/traffic_by_customer_detail.pt')
class TrafficByCustomerDetail(View):

    @property
    def actions(self):
        yield Action('Previous month', 'icon-backward',
                     'traffic_by_customer_detail', customer=self.customer_no,
                     _query=dict(month=self.previous_month()))
        yield Action('Next month', 'icon-forward',
                     'traffic_by_customer_detail', customer=self.customer_no,
                     _query=dict(month=self.next_month()))

    @property
    def title(self):
        return u'Traffic for {} in {}'.format(self.customer_no, self.month)

    @property
    def customer_no(self):
        return self.request.matchdict['customer']

    def next_month(self):
        next = self._first() + datetime.timedelta(days=31)
        return next.strftime('%Y-%m')

    def previous_month(self):
        next = self._first() - datetime.timedelta(days=1)
        return next.strftime('%Y-%m')

    @property
    def month(self):
        return self.request.params.get(
            'month', datetime.date.today().strftime('%Y-%m'))

    def _first(self):
        return datetime.date(*map(int, tuple(self.month.split('-')) + (1,)))

    def _last(self):
        first = self._first()
        next_month = first + datetime.timedelta(31)
        first_next = next_month.replace(day=1)
        return first_next - datetime.timedelta(days=1)

    def traffic_by_rg(self):
        db = zope.component.getUtility(IDatabase)
        query = db.query(Traffic).join(ResourceGroup).\
            filter(ResourceGroup.customer_no == self.customer_no,
                   Traffic.day >= self._first(),
                   Traffic.day <= self._last()).\
            group_by(Traffic.resource_group_id).\
            add_columns(func.sum(Traffic.traffic))
        return query

    def total(self):
        db = zope.component.getUtility(IDatabase)
        query = db.query(Traffic).join(ResourceGroup).\
            filter(ResourceGroup.customer_no == self.customer_no,
                   Traffic.day >= self._first(),
                   Traffic.day <= self._last()).\
            group_by(ResourceGroup.customer_no).\
            add_columns(func.sum(Traffic.traffic))
        try:
            return query.one()[1]
        except NoResultFound:
            return 0


@view_config(route_name='traffic_chart_data')
def traffic_data(request):
    traffic = Traffic.query().filter(
        Traffic.day == datetime.date(2012, 9, 10)).all()

    rgs = {}
    for record in traffic:
        rgs.setdefault(record.resource_group, 0)
        rgs[record.resource_group] += record.traffic

    result = {'name': 'traffic', 'children': []}
    for rg, traffic in rgs.items():
        result['children'].append(
            dict(name=rg.name, size=traffic))

    return Response(json.dumps(result), content_type='application/json')


@view_config(route_name='traffic_chart',
             renderer='templates/traffic_chart.pt')
class TrafficChart(View):

    title = 'Traffic chart'

    @property
    def customers(self):
        result = {}
        for resource_group in ResourceGroup.query():
            # XXX modelling customers would make this better
            result.setdefault(resource_group.customer_no, [])
            result[resource_group.customer_no].append(resource_group)
        return result
