import gocept.directory2.api.testing
import xmlrpclib


class AuthTests(gocept.directory2.api.testing.XMLRPCTestCase):
    """Testing authentication and authorization."""

    layer = gocept.directory2.api.testing.HTTP_LAYER

    def create_node(self, ring=None):
        from gocept.directory2.machine import Node
        import transaction
        node = Node.create(
            name='asdf', directory_password='qwe', directory_ring=ring,
            resource_group=self.layer['rg0'], location=self.layer['loc0'])
        transaction.commit()
        return node

    def test_not_authenticated_user_cannot_access_ring0(self):
        api = self.xmlrpc('/directory')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.check_consistency()
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_not_existing_machine_name_prevents_access(self):
        self.create_node(ring=0)
        api = self.xmlrpc('/directory', 'foo', 'qwe')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.check_consistency()
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_wrong_password_prevents_access(self):
        self.create_node(ring=0)
        api = self.xmlrpc('/directory', 'asdf', 'baz')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.check_consistency()
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_ring0_machine_can_access_ring0(self):
        self.create_node(ring=0)
        api = self.xmlrpc('/directory', 'asdf', 'qwe')
        with self.assertNothingRaised():
            api.check_consistency()

    def test_ring1_machine_cannot_access_ring0(self):
        self.create_node(ring=1)
        api = self.xmlrpc('/directory', 'asdf', 'qwe')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.check_consistency()
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_ring1_machine_can_access_ring1_in_its_resource_group(self):
        self.create_node(ring=1)
        api = self.xmlrpc('/directory/rg0', 'asdf', 'qwe')
        with self.assertNothingRaised():
            api.lookup_node('asdf')

    def test_not_authenticated_user_cannot_access_ring1(self):
        self.create_node(ring=1)
        api = self.xmlrpc('/directory/rg0')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.lookup_node('asdf')
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_ring0_machine_cannot_access_ring1(self):
        self.create_node(ring=0)
        api = self.xmlrpc('/directory/rg0', 'asdf', 'qwe')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.lookup_node('asdf')
        self.assertEndsWith('403 Forbidden>', str(err.exception))

    def test_ring1_machine_cannot_access_ring1_in_other_resource_group(self):
        self.create_node(ring=1)
        api = self.xmlrpc('/directory/services', 'asdf', 'qwe')
        with self.assertRaises(xmlrpclib.ProtocolError) as err:
            api.lookup_node('asdf')
        self.assertEndsWith('403 Forbidden>', str(err.exception))
