from sqlalchemy import Column, Integer, Text, ForeignKey
from sqlalchemy.orm.collections import attribute_mapped_collection
import sqlalchemy.dialects.postgresql
import gocept.directory2.db
import sqlalchemy.orm
import netaddr


class IPAddress(gocept.directory2.db.Object):
    """An IP address."""

    # XXX twin network behaviour?

    __tablename__ = 'ipaddresses'

    network_id = Column(Integer, ForeignKey('networks.id'), nullable=False)
    network = sqlalchemy.orm.relation(
        'Network', backref=sqlalchemy.orm.backref('addresses',
        collection_class=attribute_mapped_collection('address')))

    nic_id = Column(Integer, ForeignKey('nics.id'), nullable=True)
    nic = sqlalchemy.orm.relation(
        'NIC', uselist=False,
        backref=sqlalchemy.orm.backref('addresses'))

    address = Column(sqlalchemy.dialects.postgresql.INET, unique=True)
    reverse = Column(Text)

    @classmethod
    def create(cls, **kw):
        if 'address' not in kw:
            raise KeyError('Need IP address to create address object.')
        existing_address = IPAddress.query().filter(
            IPAddress.address == kw['address'])
        if existing_address.count():
            return existing_address.one()
        return super(IPAddress, cls).create(**kw)

    @property
    def cidr(self):
        return '%s/%s' % (self.address, self.network.netaddr.prefixlen)

    @property
    def index(self):
        return self.netaddr.value - self.network.netaddr.first

    @property
    def netaddr(self):
        return netaddr.IPAddress(self.address)

    @property
    def in_use(self):
        return self.nic is not None

    # XXX __cmp__
