# vim: set fileencoding=utf-8 :
# flake8: noqa
from pyramid_mailer.interfaces import IMailer
from pyramid_mailer.message import Message
import datetime
import logging
import uuid
import zope.component


logger = logging.getLogger('gocept.directory')

RFC2822_DATE_FORMAT = '%a, %d %b %Y %H:%M:%S +0000'

MAIL_TEMPLATE = dict(en=u"""Hello,

%(content)s

Kind regards,
your gocept.net system administrators

-- 
gocept.net - build out your app!
http://www.gocept.com - phone +49 345 122 9889 0 - support@gocept.com
Python, Pyramid, Plone, Zope - consulting, development, hosting
""",
                     de=u"""Hallo,

%(content)s

Mit freundlichen Grüßen,
Ihre gocept.net System-Administratoren

-- 
gocept.net - build out your app!
http://www.gocept.com - phone +49 345 122 9889 0 - support@gocept.com
Python, Pyramid, Plone, Zope - consulting, development, hosting
""")


def sendmail(log_origin, recipients, lang, subject, content):
        # Mark mails as being automatically generated according to RfC 3834 by
        # sending an "Auto-Submitted: auto-generated" header line, but
        # ignore the recommendation to use an empty Return-Path address.
        # Also, send a "Precedence: bulk" header line as a hint to our own
        # outgoing mail server.
        body = MAIL_TEMPLATE[lang] % dict(content=content)
        headers = {}
        headers['Message-ID'] = '<%s@gocept.net>' % uuid.uuid1()
        headers['Date'] = datetime.datetime.utcnow().strftime(
            RFC2822_DATE_FORMAT)
        headers['Auto-Submitted'] = 'auto-generated'
        headers['Precedence'] = 'bulk'

        message = Message(subject=subject,
                          sender='gocept support <support@gocept.com>',
                          recipients=recipients,
                          body=body,
                          extra_headers=headers)

        mailer = zope.component.getUtility(IMailer)
        mailer.send(message)
        logger.info('Queueing mail for %s (%s) to %s' % (
                    log_origin, headers['Message-ID'], recipients))
