#coding:utf8
from ..domain import Domain, User
import unittest
import gocept.directory2.testing
import transaction


class DomainTest(unittest.TestCase):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def setUp(self):
        super(DomainTest, self).setUp()
        self.user1 = User.create(id=100)
        self.uid = self.user1.id

    def test_create_domain(self):
        domain1 = Domain.create(name='gocept.com')
        domain1.uid = self.uid
        domain = Domain.query().all()[0]
        self.assertEqual('gocept.com', domain.name)

    def test_domain_user_relation(self):
        domain1 = Domain.create(name='gocept.com')
        domain1.uid = self.uid
        domain = Domain.query().all()[0]
        self.assertEqual(self.user1, domain.user)

    def test_user_domains_backref(self):
        domain1 = Domain.create(name='gocept.com')
        domain2 = Domain.create(name='flyingcircus.io')
        domain1.uid = self.uid
        domain2.uid = self.uid
        user = User.query().all()[0]
        self.assertEqual([domain1, domain2], user.domains)

    def test_domain_id_sequence(self):
        domain1 = Domain.create(name='gocept.com', uid=self.uid)
        transaction.commit()
        self.assertEqual(1, domain1.id)
        domain2 = Domain.create(name='foobar.com', uid=self.uid)
        self.assertIsNone(domain2.id)
        transaction.commit()
        self.assertIsNotNone(domain2.id)

    def test_subdomains_relation(self):
        domain1 = Domain.create(id=1, name='gocept.com', uid=self.uid)
        Domain.create(
            id=2, name='www.gocept.com', subdom_of=domain1.id, uid=self.uid)
        Domain.create(
            id=3, name='admin.gocept.com', subdom_of=domain1.id, uid=self.uid)
        domain1, subdomain1, subdomain2 = Domain.query().order_by(
            Domain.id).all()
        self.assertEqual([subdomain1, subdomain2], domain1.subdomains)
        self.assertEqual(domain1, subdomain1.subdomain_of)
        self.assertEqual(domain1, subdomain2.subdomain_of)

    def test_idn_name_is_autogenerated_when_needed(self):
        domain = Domain.create(id=1, name=u'müller.de', uid=self.uid)
        self.assertEqual('xn--mller-kva.de', domain.idn_name)

    def test_can_handle_utf8_strings(self):
        transaction.commit()
        # simulation old data from the PHP domainshuttle script
        self.layer.db_domainshuttle.session.execute(
            u"INSERT into domain (id, uid, name) VALUES "
            u"(1, %s, 'türöffner-halle.de');" % self.uid)
        self.assertEqual(
            u'türöffner-halle.de',
            Domain.query().filter(Domain.id == 1).all()[0].name)

    def test_idn_name_is_not_generated_when_not_needed(self):
        domain = Domain.create(id=1, name=u'mueller.de', uid=self.uid)
        self.assertEqual(None, domain.idn_name)
