from ..numberrange import NumberRange, NumberRangeFactory
import gocept.directory2.testing
import unittest


@NumberRangeFactory
class ExampleNumberRange(NumberRange):

    type = 'example'
    min = 10
    max = 14

    used = [10, 12]

    def is_available(self, number):
        if number in self.used:
            return False
        return True


class NumberRangeFTests(unittest.TestCase):
    """Functional tests for ..numberrange.NumberRange*"""

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def test_Factory_creates_numberranges_on_the_fly_and_stores_data(self):
        nr = ExampleNumberRange()
        self.assertEqual(10, nr.next_number)
        self.assertEqual('example', nr.type_)

    def test_Factory_returns_always_the_same_object(self):
        self.assertTrue(ExampleNumberRange() is ExampleNumberRange())

    def test_next_returns_different_number_for_each_call_but_only_unused(self):
        next = ExampleNumberRange().next
        self.assertEqual(11, next())
        self.assertEqual(13, next())
        self.assertEqual(14, next())
        self.assertEqual(11, next())  # wrap around

    def test_raises_RuntimeError_if_number_range_is_exausted(self):
        numberrange = ExampleNumberRange()
        numberrange.used = range(10, 15)
        with self.assertRaises(RuntimeError) as err:
            numberrange.next()
        self.assertEqual("'example' space exhausted.", str(err.exception))
