from ..permission import Permission
from ..resourcegroup import ResourceGroup
import gocept.directory2.testing
import gocept.testing.assertion
import unittest


class PermissionIdConstraints(unittest.TestCase,
                              gocept.testing.assertion.Exceptions):

    layer = gocept.directory2.testing.BASE_APPLICATION_LAYER

    def test_id_number_validator_allows_storing_a_perm_with_its_own_id(self):
        perm = Permission.create(id_number=29)
        with self.assertNothingRaised():
            perm.id_number = 29

    def test_permission_ids_must_be_unique(self):
        Permission.create(id_number=29)
        with self.assertRaises(AssertionError) as e:
            Permission.create(id_number=29)
        self.assertEqual(
            'Permission id already used for an existing permission.',
            str(e.exception))

    def test_permission_ids_must_be_unique_including_group_ids(self):
        rg = ResourceGroup.create()
        with self.assertRaises(AssertionError) as e:
            Permission.create(id_number=rg.id_number)
        self.assertEqual(
            'Permission id already used for an existing resource group.',
            str(e.exception))
