import datetime
import pytz


def now():
    # Provide a mockable, TZ-aware implementation
    now = datetime.datetime.utcnow()
    return pytz.UTC.localize(now)


def ranges_overlap(a, b):
    # Ensure the lower range comes first
    first, second = sorted([a, b])
    # Now, we do overlap if the first end is past the second start
    return not first[1] <= second[0]


def format_interval(interval):
    result = []
    if interval.days:
        days_s = '%s day' % interval.days
        if interval.days > 1:
            days_s += 's'
        result.append(days_s)
    hours = interval.seconds / (60*60)
    if hours:
        hours_s = '%s hour' % hours
        if hours > 1:
            hours_s += 's'
        result.append(hours_s)
    minutes = (interval.seconds - (hours*60*60))/60
    if minutes:
        minutes_s = '%s minute' % minutes
        if minutes > 1:
            minutes_s += 's'
        result.append(minutes_s)
    return ', '.join(result)
